from __future__ import print_function
import sys
import os

from ..singleton.singleton import Singleton
from ..configurator.configurator import Configurator
from ..logger.logger import Logger
import requests
import time


class Connector(object):
    __metaclass__ = Singleton
    __conf = Configurator()
    __host = __conf.getConfig()['host'] # noqa #nosec
    __token = "" # noqa #nosec
    __tokenType = "" # noqa #nosec
    __expires = ""
    logger = Logger()

    def __init__(self):
        self.logger.log(2, 'connector.__init__', 'Initing connector...')
        conf = self.__conf.getConfig()
        self.__host = conf['host']

        self.__verifyHttps = conf['verifyHttps']
        self.session = requests.Session()
        self.logger.log(2, 'connector.__init__', 'Host:'+self.__host)
        self.logger.log(1, 'connector.__init__', 'Connector initialized')

    def initialize(self):
        if self.__host == '' or self.__token == '':
            return False
        try:
            self.getNewToken()
        except Exception as e:
            print('Exception while getNewToken ', e, file=sys.stderr)
            return False
        return True

    def __checkTokenExpiration(self):
        # returns true of ok, false if expired
        self.logger.log(2, 'connector.__checkTokenExpiration',
                        'Token is valid: ' +
                        str(self.__expires > time.time()))
        if self.__expires < time.time() or self.__token == "":
            self.logger.log(2, 'connector.__checkTokenExpiration',
                            'Token is invalid because ' + str(self.__expires) +
                            '>' + str(time.time()))
            return False
        print("ret true - token ok", file=sys.stderr)
        return True

    def __updateToken(self):
        pass

    def getNewToken(self):
        self.logger.log(1, 'connector.__getNewToken', 'Retrieving new token')

        conf = self.__conf.getConfig()
        self.__host = conf['host']
        credentials = self.__conf.getCreddentials()
        self.__verifyHttps = conf['verifyHttps']

        data = {'grant_type': 'password',
                'client_id': 'invea-tech',
                'username': credentials['username'],
                'password': credentials['password']}

        try:
            self.logger.log(1, 'connector.__getNewToken', 'Sending request')
            self.session = requests.Session()
            self.session.proxies = self.getProxy()
            self.logger.log(2, 'connector.__getNewToken',
                            'proxy={}'.format(self.getProxy()))
            self.logger.log(2, 'connector.__getNewToken',
                            'verify={}'.format(self.getCertificate()))
            r = self.session.post(self.__host + '/resources/oauth/token', # noqa # nosec
                                  data=data, verify=self.getCertificate())
            if r.status_code == 200:
                self.logger.log(1, 'connector.__getNewToken',
                                'Response {}'.format(r.status_code))
            else:
                self.logger.log(1, 'connector.__getNewToken',
                                'Response {} {}'.format(r.status_code, r.text))

            self.__token = r.json()['access_token']
            self.__tokenType = r.json()['token_type']
            self.__expires = time.time() + r.json()['expires_in'] - 60

            self.logger.log(2, 'connector.__getNewToken', 'New token: ' +
                            self.__token + ' Expires: ' + str(self.__expires))
            self.logger.log(1, 'connector.__getNewToken',
                            'New token successfully retrieved')
            return "ok", 200
        except Exception as e:
            self.logger.log(1, 'connector.__getNewToken',
                            'ERROR while retrieve token {0}'.format(e))
            return "{0}".format(e), 400

    def sendGet(self, url, params="", data=""):
        conf = self.__conf.getConfig()

        self.__host = conf['host']
        self.__verifyHttps = conf['verifyHttps']

        if self.__checkTokenExpiration() is False:
            # print('Token expired ', file=sys.stderr)
            self.getNewToken()

        self.logger.log(1, 'connector.sendGet', 'Sending GET to '+url)
        self.logger.log(2, 'connector.sendGet', 'Parameters: ' +
                        str(params) + ' Data: ' + str(data))

        headers = {'Authorization': self.__tokenType + ' ' + self.__token}
        self.session = requests.Session()
        self.session.proxies = self.getProxy()
        self.logger.log(2, 'connector.sendGet',
                        'proxy={}'.format(self.getProxy()))
        self.logger.log(2, 'connector.sendGet',
                        'verify={}'.format(self.getCertificate()))
        r = self.session.get(self.__host + url, params=params, data=data, # noqa # nosec
                         headers=headers, verify=self.getCertificate())

        self.logger.log(1, 'connector.sendGet', 'GET request to ' + r.url +
                        ' returned with code ' + str(r.status_code))
        if r.status_code == 500:
            self.logger.log(1, 'connector.sendGet',
                            'GET request finished with ERROR: ' +
                            str(r.text))
        try:
            return r.json(), r.status_code
        except Exception:
            return {}, r.status_code

    def sendPost(self, url, params="", data=""):
        conf = self.__conf.getConfig()

        self.__host = conf['host']
        self.__verifyHttps = conf['verifyHttps']

        # check if token expired. If yes, update it
        if self.__checkTokenExpiration() is False:
            self.getNewToken()

        self.logger.log(1, 'connector.sendPost', 'Sending POST to ' + url)
        self.logger.log(2, 'connector.sendPost', 'Parameters: ' +
                        str(params) + ' Data: ' + str(data))

        headers = {'Authorization': self.__tokenType + ' ' + self.__token}
        self.session = requests.Session()
        self.session.proxies = self.getProxy()
        self.logger.log(2, 'connector.sendPost',
                        'proxy={}'.format(self.getProxy()))
        self.logger.log(2, 'connector.sendPost',
                        'verify={}'.format(self.getCertificate()))
        r = self.session.post(self.__host + url, params=params, data=data, # noqa # nosec
                          headers=headers, verify=self.getCertificate())

        self.logger.log(1, 'connector.sendGet', 'POST request to ' +
                        r.url + ' returned with code ' + str(r.status_code))

        try:
            return r.json(), r.status_code
        except Exception:
            return {}, r.status_code

    def getCertificate(self):
        result = False

        conf = self.__conf.getConfig()
        self.__verifyHttps = conf['verifyHttps']

        if self.__verifyHttps is False:
            return result
        if os.path.isfile('store/cert.pem'):
            if os.stat("store/cert.pem").st_size > 3:
                result = "store/cert.pem"

        return result

    def getProxy(self):
        conf = self.__conf.getConfig()
        proxies = None
        if not conf['proxy']['enabled']:
            return proxies

        creds_str = ""
        if conf['proxy']['username']:
            creds_str = "{0}:{1}@"

        proxy_split = conf['proxy']['address'].split("://")
        proxy_val = "{}{}:{}".format(creds_str, proxy_split[1],
                                     conf['proxy']['port'])

        proxies = {
            'http': proxy_val,
            'https': proxy_val,
        }

        return proxies
