from __future__ import print_function
import sys
import zipfile
import os

__author__ = 'IBM'

from datetime import datetime, timedelta
import json
import re
import string
import random

from flask import render_template, request, Response, send_file
from qpylib import qpylib
from app import app

from configurator.configurator import Configurator
from connector.connector import Connector
from logger.logger import Logger
from flask_wtf.csrf import CSRFProtect

logger = Logger()
conn = Connector()
conf = Configurator()

if conf.getConfig()['host'] != "":
    conn.getNewToken()

if conf.getConfig()['secretKey'] == "":
    alphabet = string.ascii_letters + string.digits
    while True:
        password = ''.join(random.choice(alphabet) for i in range(20)) # noqa #nosec
        if (any(c.islower() for c in password)
                and any(c.isupper() for c in password)
                and sum(c.isdigit() for c in password) >= 3):
            break
    conf.setSecretKey(password)

app.secret_key = conf.getConfig()['secretKey']
app.config.update(
    SESSION_COOKIE_NAME='FlowmonApp-Session',
)
csrf = CSRFProtect(app)
csrf.init_app(app)


@app.before_request
def check_csrf():
    csrf.protect()


# because python2.6 doesn't have timedelta.total_seconds()
def timedelta_total_seconds(tdelta):
    return (tdelta.microseconds + 0.0 + (tdelta.seconds + tdelta.days * 24 *
                                         3600) * 10 ** 6) / 10 ** 6


# HTML routes
@app.route('/')
@app.route('/index')
def index():
    # print("Request to index. Getting token:", conn.initialize(),
    # file=sys.stderr)
    if conn.initialize() is False:
        return render_template("notAuthorized.html")
    with open('manifest.json' if qpylib.is_sdk()
              else 'app/manifest.json') as m:
        version = json.load(m)['version']
    return render_template("index.html", version=version)


@app.route('/events')
def get_events_page():
    return render_template("events.html")


@app.route('/flows')
def get_flows_page():
    return render_template("flows.html")


@app.route('/dashboard')
def get_dashboard():
    return render_template("dashboard.html")


@app.route('/configuration')
def get_configuration():
    return render_template("configuration.html")


# service routes
@app.route('/config')
def get_conf():
    # with open('store/configuration.json') as conFile:
    #     config = json.load(conFile)
    # return json.dumps(config)
    config = Configurator()
    data = config.getConfig()
    data['proxy'].pop('password')
    return json.dumps(data)


@app.route('/config', methods=["POST"])
def set_conf():
    new_conf = request.json
    # with open('store/configuration.json', 'w') as conFile:
    #     json.dump(newConf, conFile)
    # return "ok"
    config = Configurator()
    return config.setConfig(new_conf)


@app.route('/config_cert', methods=["POST"])
def set_conf_cert():
    if 'file' not in request.files:
        return "Missing cert file", 500

    f = request.files['file']
    fileName = "cert.pem"
    f.save("store/{0}".format(fileName))

    logger.log(1, 'endpoint config_cert',
               'Stored cert file {0} '.format(request.files['file']))

    with open("store/{0}".format(fileName)) as pemFIle:
        lines = pemFIle.readlines()

    final = []
    for line in lines:
        line = line.rstrip()
        if len(line) < 64:
            final.append(line)
        else:
            for chunk in [line[i:i + 64] for i in range(0, len(line), 64)]:
                final.append(chunk)

    with open("store/{0}".format(fileName), "w") as pemFile:
        pemFile.write("\r\n".join(final))
        pemFile.write("\r\n")

    return "Success", 200


@app.route('/output_definitions')
def out_def():
    with open('store/outputDefinitions.json') as defs_file:
        defs = json.load(defs_file)
    return json.dumps(defs)


# api routes
@app.route('/api/perspectives')
def get_perspectives():
    resp = conn.sendGet('/rest/ads/perspectives')[0]
    return json.dumps(resp)


@app.route('/api/filters')
def get_filters():
    resp = conn.sendGet('/rest/ads/filters')[0]
    return json.dumps(resp)


@app.route('/api/methods')
def get_methods():
    resp = conn.sendGet('/rest/ads/methods')[0]
    return json.dumps(resp)


@app.route('/api/netflowSources')
def get_flowsources():
    resp = conn.sendGet('/rest/ads/net-flow-sources')[0]
    return json.dumps(resp)


@app.route('/api/events')
def get_events():
    from_time = request.args.get('from')
    to_time = request.args.get('to')
    source = request.args.get('source')
    target = request.args.get('target')
    type_value = request.args.get('type')
    filter_value = request.args.get('filter')
    persp = request.args.get('perspective')
    priority = request.args.get('priority')
    nfsource = request.args.get('nfsource')
    # create search object for flowmon api
    search = {'search': '{"from":"' + from_time + '", "to":"' + to_time + '"}'}
    if source is not None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"source": "' + source + '"}'

    if target is not None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"target": "' + target + '"}'

    if type_value is not None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"type": ' + type_value + '}'

    if filter_value is not None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"filter": ' + filter_value + '}'

    if persp is not None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"perspective": ' + persp + '}'

    if priority is not None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"priority": [' + priority + ']}'

    if nfsource is not None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"nfSource": ' + nfsource + '}'

    # print("Search: ", search, file=sys.stderr)

    search['limit'] = 10000

    resp = conn.sendGet('/rest/ads/events', search)[0]
    respjson = json.dumps({'data': resp})
    logger.log(1, 'endpoint getEvents', 'Received ' + str(len(resp)) +
               ' events')
    logger.log(2, 'endpoint getEvents', 'Received events: ' + str(resp))
    return respjson


@app.route('/api/events/<id>')
def get_event_details(id):
    event_id = request.args.get('eventID')
    logger.log(1, 'endpoint getEventDetails',
               'Retrieving event details for event ' + str(event_id))
    resp = conn.sendGet('/rest/ads/event/' + id)[0]
    return json.dumps(resp)


@app.route('/api/events/<id>/flows')
def get_event_flows(id):
    logger.log(1, 'endpoint getEventFlows',
               'Retrieving flows for event ' + str(id))
    config = Configurator()
    event_id = id
    resp = conn.sendGet('/rest/ads/event/' + event_id)[0]
    # all_profiles = json.loads(get_profiles())
    # allChannels = []
    # print('resp:', resp,  file=sys.stderr)
    # for prof in allProfiles['data']:
    # print('prof :', prof, file=sys.stderr)
    # for ch in prof['channels']:
    # print('ch :', ch, file=sys.stderr)
    # allChannels.append(ch['id'])
    # print('allChannels :', allChannels, file=sys.stderr)
    # intersectChannels = [val for val in allChannels
    # if val in resp['flowsSearch']['channels']]
    # print("resp['flowsSearch']:", resp['flowsSearch'], file=sys.stderr)
    # print('\nintersectChannels :', intersectChannels, file=sys.stderr)
    search = {
        "from": resp['flowsSearch']['from'],
        "to": resp['flowsSearch']['to'],
        "profile": resp['flowsSearch']['profile'],
        "filter": resp['flowsSearch']['filter'],
        # "channels": resp['flowsSearch']['channels'],
        "channels": resp['flowsSearch']['channels'],
        "location": resp['flowsSearch']['location']
    }
    print("searchObj:", search, file=sys.stderr)
    output = config.getConfig()['flowFormatInternal']

    # print('=======', file=sys.stderr)
    # print(search, file=sys.stderr)
    resp = conn.sendPost('/rest/fmc/analysis/flows',
                         data={"search": json.dumps(search),
                               "output": json.dumps(output),
                               "showonly": 10000})
    # print('result', resp, file=sys.stderr)

    if resp[1] != 200:
        error_msg = resp[0]['message']
        pattern = re.compile("^Profile channel (.*) not found in profile .*$")
        regex_match = pattern.match(error_msg)
        if regex_match:
            return " Error: Misconfigured channel " + ','.join(
                str(i) for i in regex_match.groups()) + \
                   " on Flowmon appliance.", 500
        return " Error: " + str(resp[0]['message']), 500

    try:
        search_id = resp[0]['id']
        logger.log(1, 'endpoint getEventFlows',
                   'Flow search id: ' + str(search_id))
        logger.log(1, 'endpoint getEventFlows',
                   'Checking if flow search has finished')
        resp = conn.sendGet('/rest/fmc/analysis/results/' + str(search_id))

        while True:
            if resp[1] == 200:
                logger.log(2, 'endpoint getEventFlows',
                           'Flow search has finished successfully')
                break
            logger.log(2, 'endpoint getEventFlows',
                       'Checking if flow search has finished')
            resp = conn.sendGet('/rest/fmc/analysis/results/' + str(search_id))

        # print('======= ',resp[1], file=sys.stderr)
        logger.log(1, 'endpoint getEventFlows',
                   'Received ' + str(len(resp[0])) + ' flows')
        logger.log(2, 'endpoint getEventFlows',
                   'Received flows: ' + str(resp[0]))
        return json.dumps(resp[0])
    except Exception:
        logger.log(1, 'endpoint getEventFlows',
                   'Flow search has finished with ERROR: ' +
                   str(resp['message']))
        return " Error: " + str(resp['message']), 500


@app.route('/api/profiles')
def get_profiles():
    resp = conn.sendPost('/rest/fmc/getprofiles')[0]
    return json.dumps({'data': resp})


@app.route('/api/flows')
def get_flows():
    logger.log(1, 'endpoint getFlows', 'Retrieving flows')
    config = Configurator()
    from_time = request.args.get('from')
    to_time = request.args.get('to')
    profile = request.args.get('profile')
    channels = request.args.get('channels')
    listing = request.args.get('listing')
    filter_value = request.args.get('filter')
    count = request.args.get('count')
    search = {
        "from": from_time,
        "to": to_time,
        "profile": profile,
        "channels": channels.split(','),
        "listing": listing,
        "filter": filter_value
    }
    # with open('store/configuration.json') as conFile:
    #     config = json.load(conFile)
    # output = ["ts","td","pr","sa","sp","da","dp","flg",
    # "tos","pkt","byt","pps","bps","bpp","fl"]
    # output = config['flowFormat']
    output = config.getConfig()['flowFormatInternal']
    # print(search, file=sys.stderr)

    resp = conn.sendPost('/rest/fmc/analysis/flows',
                         data={"search": json.dumps(search),
                               "output": json.dumps(output),
                               "showonly": int(count)})
    # print('======= ', resp, file=sys.stderr)
    try:
        search_id = resp[0]['id']
        logger.log(1, 'endpoint getEventFlows',
                   'Flow search id: ' + str(search_id))
        logger.log(1, 'endpoint getFlows',
                   'Checking if flow search has finished')
        resp = conn.sendGet('/rest/fmc/analysis/results/' + str(search_id))
        while True:
            if resp[1] == 200:
                logger.log(1, 'endpoint getFlows',
                           'Flow search has finished successfully')
                break
            if resp[1] == 204:
                logger.log(1, 'endpoint getFlows',
                           'Flow search has finished successfully, ' +
                           'but there are no flows')
                return json.dumps({}), 204
            logger.log(2, 'endpoint getFlows',
                       'Checking if flow search has finished')
            resp = conn.sendGet('/rest/fmc/analysis/results/' + str(search_id))
            # print('======= ', resp[0],resp[1], file=sys.stderr)

        logger.log(1, 'endpoint getFlows', 'Received ' + str(len(resp[0])) +
                   ' flows')
        logger.log(2, 'endpoint getFlows', 'Received flows: ' + str(resp[0]))
        return json.dumps(resp[0])
    except Exception:
        print('endpoint getEventFlows', 'Error: ' + str(resp), file=sys.stderr)
        if 'syntax error' in resp[0]['message']:
            logger.log(1, 'endpoint getEventFlows',
                       'Flow search has finished with ERROR: ' +
                       'Bad filter (syntax error)')
            return "Error: " + "Bad filter", 400

        logger.log(1, 'endpoint getEventFlows',
                   'Flow search has finished with ERROR: '
                   + str(resp[0]['message']))
        return "Error: " + str(resp[0]['message']), 500


@app.route('/api/graph/type')
def graph_type():
    aggr_parts = 30
    from_time = request.args.get('from')
    to_time = request.args.get('to')
    source = request.args.get('source')
    target = request.args.get('target')
    persp = request.args.get('perspective')
    priority = request.args.get('priority')
    nfsource = request.args.get('nfsource')
    filter_value = request.args.get('filter')
    type_value = request.args.get('type')  # events, sources, targets

    logger.log(1, 'endpoint graph', 'Starting graph routine')
    logger.log(2, 'endpoint graph',
               'Parameters: from=' + str(from_time) + ' to=' + str(to_time)
               + ' source=' + str(source) + ' target=' + str(target) +
               ' perspective=' + str(persp) + ' priority=' + str(priority) +
               ' nfsource=' + str(nfsource) + ' filter=' +
               str(filter_value) + ' type=' + type_value)

    frDT = datetime.strptime(from_time, '%Y-%m-%d %H:%M')
    toDT = datetime.strptime(to_time, '%Y-%m-%d %H:%M')

    delta = timedelta_total_seconds(toDT - frDT)

    aggrDelta = int(delta / aggr_parts)  # size of one aggrPart in seconds,
    # tzoffset = time.timezone

    logger.log(2, 'endpoint graph',
               'Counted aggregation delta in seconds: ' + str(aggrDelta))

    # get events
    search = {'limit': 10000,
              'search': '{"from":"' + from_time + '","to":"' + to_time + '"}'}
    if source is not None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"source": "' + source + '"}'
    if target is not None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"target": "' + target + '"}'
    if persp is not None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"perspective": ' + persp + '}'
    if priority is not None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"priority": [' + priority + ']}'
    if nfsource is not None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"nfSource": ' + nfsource + '}'
    if filter_value is not None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"filter": ' + filter_value + '}'

    # print('   ***  Search object for graph data: ', search, file=sys.stderr)
    events, status = conn.sendGet('/rest/ads/events', search)
    if status == 500:
        if events['message'] == 'Token was not found.':
            return 'Error with token. Please reload application page', 427
    logger.log(1, 'endpoint graph', 'Received ' + str(len(events)) + ' events')
    result = []
    for i in range(aggr_parts):
        result.append({
            'from': (frDT + timedelta(0, aggrDelta * i)).strftime(
                "%Y/%m/%d %H:%M:%S"),
            'to': (frDT + timedelta(0, aggrDelta * (i + 1))).strftime(
                "%Y/%d/%m %H:%M:%S"),
            'stat': {}
        })
        # print(result[len(result)-1], file=sys.stderr)
    logger.log(2, 'endpoint graph',
               'Created graph stats array: ' + str(result))

    usedTypes = []

    if len(events) == 0:
        return json.dumps({'data': []})

    print("Result after creation: ", result, file=sys.stderr)
    i = 1
    lenEvents = len(events)

    for event in events:
        logger.log(2, 'endpoint graph',
                   'Event ' + str(i) + '/' +
                   str(lenEvents) + ': ' + str(event))
        if 'timeLast' not in event:
            event['timeLast'] = event['time']
            # event['timeLast'] = "2020-03-19 18:41:06"

        # oldFlag = re.search(reOld, event['time'])
        # newFlag = re.search(reNew, event['time'])

        splitRes = event['time'].split(".")
        if len(splitRes) > 1:
            eventTime = datetime.strptime(event['time'],
                                          '%Y-%m-%d %H:%M:%S.%f')
        else:
            eventTime = datetime.strptime(event['time'], '%Y-%m-%d %H:%M:%S')

        splitResLast = event['timeLast'].split(".")
        if len(splitResLast) > 1:
            eventTimeLast = datetime.strptime(event['timeLast'],
                                              '%Y-%m-%d %H:%M:%S.%f')
        else:
            eventTimeLast = datetime.strptime(event['timeLast'],
                                              '%Y-%m-%d %H:%M:%S')

        # print("EVENT TIMES: {0} {1}".format(eventTime, eventTimeLast),
        # file=sys.stderr)

        offsetFromStart = eventTime - frDT
        offsetFromStart = timedelta_total_seconds(offsetFromStart)
        index = int(((offsetFromStart / aggrDelta)))

        offsetFromStartLast = eventTimeLast - frDT
        offsetFromStartLast = timedelta_total_seconds(offsetFromStartLast)
        indexLast = int(((offsetFromStartLast / aggrDelta)))

        # print("EVENT: {0} {1}".format(index, indexLast), file=sys.stderr)
        logger.log(2, 'endpoint graph', 'Event ' + str(i) + '/' +
                   str(lenEvents) + ': eventTime=' + str(eventTime) +
                   ' offsetFromStart=' + str(offsetFromStart) +
                   ' index=' + str(index))

        # print("--  Event type:", event['type'], " Event time: ",
        # event['time'], " Index: ",
        # index, file=sys.stderr)
        # print("{0}".format(json.dumps(event)), file=sys.stderr)

        if type_value == 'events':
            for i in range(index, indexLast + 1):
                if i < 0:
                    continue
                if i >= 30:
                    break
                if event['type'] in result[i]['stat']:
                    # print("incr", file=sys.stderr)
                    result[i]['stat'][event['type']] += 1
                else:
                    # print(" new", file=sys.stderr)
                    result[i]['stat'][event['type']] = 1

        elif type_value == 'sources':
            for i in range(index, indexLast + 1):
                if i < 0:
                    continue
                if i >= 30:
                    break
                if event['type'] in result[i]['stat']:
                    # print("incr", file=sys.stderr)
                    result[i]['stat'][event['type']].append(
                        event['source']['ip'])
                else:
                    # print(" new", file=sys.stderr)
                    result[i]['stat'][event['type']] = [event['source']['ip']]

        elif type_value == 'targets':
            for i in range(index, indexLast + 1):
                if i < 0:
                    continue
                if i >= 30:
                    break
                if event['type'] in result[i]['stat']:
                    # print("incr", file=sys.stderr)
                    result[i]['stat'][event['type']] += \
                        list(map(lambda x: x['ip'], event['targets']))
                else:
                    # print(" new", file=sys.stderr)
                    result[i]['stat'][event['type']] = \
                        list(map(lambda x: x['ip'], event['targets']))

        if event['type'] not in usedTypes:
            usedTypes.append(event['type'])

        i = i + 1

    for r in result:
        for t in usedTypes:
            if t not in r['stat']:
                r['stat'][t] = 0

    if type_value == 'targets' or type_value == 'sources':
        for o in result:
            for t in o['stat']:
                if o['stat'][t] != 0:
                    o['stat'][t] = len(set(o['stat'][t]))

    return json.dumps({'data': result})


@app.route('/api/graph/top_sources')
def topsou():
    from_time = request.args.get('from')
    to_time = request.args.get('to')
    method = request.args.get('method')
    source = request.args.get('source')
    target = request.args.get('target')
    persp = request.args.get('perspective')
    priority = request.args.get('priority')
    nfsource = request.args.get('nfsource')
    filter_val = request.args.get('filter')

    logger.log(1, 'endpoint top_sources', 'Starting top_sources routine')
    logger.log(2, 'endpoint top_sources',
               'Parameters: from=' + str(from_time) + ' to=' +
               str(to_time) + ' source=' + str(source) + ' target=' +
               str(target) + ' perspective=' + str(persp) + ' priority=' +
               str(priority) + ' nfsource=' + str(nfsource) +
               ' filter=' + str(filter_val))

    search = {'limit': 10000, 'search': '{"from":"' + from_time +
                                        '","to":"' + to_time + '"}'}
    if method is not None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"type": "' + method + '"}'
    if source is not None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"source": "' + source + '"}'
    if target is not None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"target": "' + target + '"}'
    if persp is not None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"perspective": ' + persp + '}'
    if priority is not None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"priority": [' + priority + ']}'
    if nfsource is not None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"nfSource": ' + nfsource + '}'
    if filter_val is not None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"filter": ' + filter_val + '}'

    events, status = conn.sendGet('/rest/ads/events', search)
    if status == 500:
        if events['message'] == 'Token was not found.':
            return 'Error with token. Please reload application page', 427

    logger.log(1, 'endpoint top_sources', 'Received ' +
               str(len(events)) + ' events')

    stats = {}
    resolve = {}
    country = {}
    result = []
    # result = [
    #     {ip:"", count:},
    #     {ip:"", count:},
    #     {ip:"", count:},
    # ]
    i = 1
    for event in events:
        logger.log(2, 'endpoint top_sources',
                   'Event ' + str(i) + '/' + str(len(events)) + ' source IP: '
                   + str(event['source']['ip']))
        if event['source']['ip'] in stats:
            stats[event['source']['ip']] += 1
        else:
            stats[event['source']['ip']] = 1
            resolve[event['source']['ip']] = event['source']['resolved']
            country[event['source']['ip']] = event['source']['country']
        i = i + 1
    for stat in stats:
        result.append({"ip": stat, "count": stats[stat],
                       "resolved": resolve[stat],
                       "country": country[stat]})

    return json.dumps({'data': result})


@app.route('/api/graph/priority')
def graph_prior():
    return ""


@app.route('/getcontext', methods=['GET'])
def get_context():
    context = request.args.get("context")
    qpylib.log("Setting the results to: " + context)
    return json.dumps({"app_id": qpylib.get_app_id(), "context": context})


@app.route('/widget', methods=['GET'])
def get_dashboard_content():
    widget_id = request.args.get('id', type=int)
    widget_params = Configurator().getWidgetsInfo()
    widget_perspective_obj = widget_params[widget_id]

    logger.log(1, 'endpoint widget', 'Starting widget routine for widget ' +
               str(widget_id))
    logger.log(2, 'endpoint widget', 'Widget config:  params=' +
               str(widget_params) +
               ' perspective=' + str(widget_perspective_obj))

    config = Configurator()
    if config.getConfig()['host'] == "" or conn.__token == "":
        return Response(response=json.dumps({
            'id': 'dashBoardItem1',
            'title': 'Flowmon Events by priority',
            'HTML': render_template('notAuthorizedWidget.html')}),
                        status=200, mimetype='application/json')

    try:
        perspectives_resp = conn.sendGet('/rest/ads/perspectives')
    except Exception:
        return Response(response=json.dumps({
            'id': 'dashBoardItem1',
            'title': 'Flowmon Events by priority',
            'HTML': render_template('notAuthorizedWidget.html')}),
                        status=200, mimetype='application/json')
    if perspectives_resp[1] != 200:
        return Response(response=json.dumps({
            'id': 'dashBoardItem1',
            'title': 'Flowmon Events by priority',
            'HTML': render_template('notAuthorizedWidget.html')}),
                        status=200, mimetype='application/json')

    perspectives = perspectives_resp[0]

    # print('Widgets perspective', widgetPerspectiveObj['perspective_name'],
    # widgetPerspectiveObj['perspective_id'], file=sys.stderr)
    # print('Perspectives: ', perspectives, file=sys.stderr)

    selected_persp_id = widget_perspective_obj['perspective_id']

    if widget_perspective_obj['perspective_id'] == 0:  # nothing is set in file
        selected_persp_id = perspectives[0]['id']
        widget_perspective_obj['perspective_id'] = perspectives[0]['name']
        widget_perspective_obj['perspective_name'] = perspectives[0]['name']
        Configurator().setWidgetInfoById(widget_id, perspectives[0]['id'],
                                         perspectives[0]['name'])
    if widget_perspective_obj['period'] == 0:  # nothing is set in file
        widget_perspective_obj['period'] = '1'
        Configurator().setWidgetPeriodById(widget_id, '1')

    search = {}
    if widget_perspective_obj['period'] == '1':
        from_time = datetime.now() + timedelta(hours=-1)
        period_text = '1 hour'
    elif widget_perspective_obj['period'] == '3':
        from_time = datetime.now() + timedelta(hours=-3)
        period_text = '3 hours'
    elif widget_perspective_obj['period'] == '6':
        from_time = datetime.now() + timedelta(hours=-6)
        period_text = '6 hours'
    elif widget_perspective_obj['period'] == '12':
        from_time = datetime.now() + timedelta(hours=-12)
        period_text = '12 hours'
    elif widget_perspective_obj['period'] == '24':
        from_time = datetime.now() + timedelta(hours=-24)
        period_text = '24 hours'
    elif widget_perspective_obj['period'] == '72':
        from_time = datetime.now() + timedelta(days=-3)
        period_text = '3 days'

    # fr = datetime.now() + timedelta(days=-1)
    from_time = from_time.strftime("%Y-%m-%d %H:%M:%S")
    to_time = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    # print('Data for widget:',fr, to,  file=sys.stderr)
    logger.log(2, 'endpoint widget', 'Widget' + str(widget_id) +
               ' Timerange: from=' +
               str(from_time) + ' to=' + str(to_time))

    search['search'] = '{"from":"' + from_time + '", "to":"' + to_time + '"}'
    if selected_persp_id != -1:
        search['search'] = search['search'][:-1] + ', "perspective":"' + \
                           str(selected_persp_id) + '"}'
    search['limit'] = 10000
    logger.log(1, 'endpoint widget',
               'Get data for widget' + str(widget_id) + ': perspective ' +
               str(selected_persp_id) +
               "/" + widget_perspective_obj['perspective_name'] +
               " From: " + str(from_time) + " To: " + str(to_time) + " (" +
               period_text + ")")
    resp, status = conn.sendGet('/rest/ads/events', search)
    if status == 500:
        if resp['message'] == 'Token was not found.':
            return 'Error with token. Please reload application page', 427

    logger.log(1, 'endpoint widget', 'Widget' + str(widget_id) + ' received ' +
               str(len(resp)) + ' events')
    rules = ['', 'info', 'low', 'medium', 'high', 'critical']
    results = {
        'info': 0,
        'low': 0,
        'medium': 0,
        'high': 0,
        'critical': 0
    }
    for event in resp:
        # if selectedPerspId != -1:

        if not event['perspectives']:
            continue
        sec_persp = ''
        for ev_persp in event['perspectives']:
            if ev_persp['id'] == selected_persp_id:
                sec_persp = ev_persp
                break
        # print('Event perspective', secPersp, file=sys.stderr)
        if sec_persp == "":
            continue
        # print(rules[secPersp['priority']], ' += 1', file=sys.stderr)
        results[rules[sec_persp['priority']]] += 1

        # else:
        # print('ev:', ev, file=sys.stderr)
        # results[rules[ev['priority']]] += 1

    # print('Results', results, file=sys.stderr)
    # print('Total events ', len(resp), file=sys.stderr)
    logger.log(1, 'endpoint widget', 'Widget' + str(widget_id) + ' data: ' +
               str(results))

    return Response(response=json.dumps({
        'id': 'dashBoardItem1',
        'title': 'Flowmon Events by priority',
        'HTML': render_template('widget.html',
                                perspectives=perspectives,
                                perspName=widget_perspective_obj
                                ['perspective_name'],
                                period=period_text,
                                crit=results['critical'],
                                high=results['high'],
                                med=results['medium'],
                                low=results['low'],
                                info=results['info'])}),
                    status=200, mimetype='application/json', )


@app.route('/widget_test', methods=['GET'])
def ggg():
    # r = qpylib.REST('get', 'api/system/servers')
    # print('REST CALL', r, r.headers, file=sys.stderr)
    return json.dumps({'test': 'HOHOHO'})


@app.route('/widget/changePerspective', methods=['GET'])
def gggg():
    widget_id = request.args.get('id', type=int)
    new_persp_id = request.args.get('perspid', type=int)

    # print('new persp val', newPerspId, type(newPerspId), file=sys.stderr)
    perspectives = conn.sendGet('/rest/ads/perspectives')[0]
    # print('perspectives', perspectives, file=sys.stderr)
    found_persp = ''
    for persp in perspectives:
        # print('persp',persp, file=sys.stderr)
        if persp['id'] == new_persp_id:
            found_persp = persp
            break
    Configurator().setWidgetInfoById(widget_id, new_persp_id,
                                     found_persp['name'])
    return ""


@app.route('/widget/changePeriod', methods=['GET'])
def ggggg():
    widget_id = request.args.get('id', type=int)
    new_period = request.args.get('period', type=str)

    Configurator().setWidgetPeriodById(widget_id, new_period)
    return ""


@app.route('/get_log')
def getlog():
    if os.path.exists("/store/log/logs.zip"):
        os.remove("/store/log/logs.zip")

    zfile = zipfile.ZipFile("/store/log/logs.zip", "a")
    zfile.write("/store/log/startup.log")
    zfile.write("/store/log/flowmon_app.log")
    zfile.close()

    # return send_file('../store/log/flowmon_app.log', as_attachment=True,
    #                  attachment_filename='flowmon_app.log', cache_timeout=-1)

    return send_file('../store/log/logs.zip', as_attachment=True,
                     attachment_filename='logs.zip', cache_timeout=-1)
